package com.zhaohaoting.framework.recyclerview.adapter;

import android.support.v7.widget.RecyclerView;
import android.view.ViewGroup;


import com.zhaohaoting.framework.recyclerview.entity.SectionEntity;
import com.zhaohaoting.framework.recyclerview.holder.BaseQuickViewHolder;

import java.util.List;

public abstract class BaseSectionQuickAdapter<T extends SectionEntity,E, K extends BaseQuickViewHolder> extends BaseQuickAdapter<T,E, K> {


    protected int mSectionHeadResId;
    protected static final int SECTION_HEADER_VIEW = 0x00000444;

    /**
     * Same as QuickAdapter#QuickAdapter(Context,int) but with
     * some initialization data.
     *
     * @param sectionHeadResId The section head layout id for each item
     * @param layoutResId      The layout resource id of each item.
     * @param data             A new list is created out of this one to avoid mutable list
     */
    public BaseSectionQuickAdapter(RecyclerView recyclerView, int layoutResId, int sectionHeadResId, List<T> data) {
        super(recyclerView, layoutResId, data);
        this.mSectionHeadResId = sectionHeadResId;
    }

    @Override
    protected int getDefItemViewType(int position) {
        return mData.get(position).isHeader ? SECTION_HEADER_VIEW : 0;
    }

    @Override
    protected K onCreateDefViewHolder(ViewGroup parent, int viewType) {
        if (viewType == SECTION_HEADER_VIEW)
            return createBaseViewHolder(getItemView(mSectionHeadResId, parent));

        return super.onCreateDefViewHolder(parent, viewType);
    }

    @Override
    public void onBindViewHolder(K holder, int positions) {
        switch (holder.getItemViewType()) {
            case SECTION_HEADER_VIEW:
                setFullSpan(holder);
                convertHead(holder, mData.get(holder.getLayoutPosition() - getHeaderLayoutCount()));
                break;
            default:
                super.onBindViewHolder(holder, positions);
                break;
        }
    }

    protected abstract void convertHead(BaseQuickViewHolder helper, T item);

}
